@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.c,v 1.215 2005/04/04 11:16:18 relson Exp $ */
+/* $Id: bogoconfig.c,v 1.230 2005/06/29 10:05:22 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -37,6 +37,7 @@ Note: bogolexer also uses configfile.c.
 
 #include <ctype.h>
 #include <unistd.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -174,6 +175,15 @@ static char *get_string(const char *name, const char *arg)
     return s;
 }
 
+static e_txn get_txn(const char *name, const char *arg)
+{
+    e_txn t = get_bool(name, arg) ? T_ENABLED : T_DISABLED;
+    if (DEBUG_CONFIG(2))
+	fprintf(dbgout, "%s -> %s\n", name,
+		t ? "enabled" : "disabled");
+    return t;
+}
+
 void process_parameters(int argc, char **argv, bool warn_on_error)
 {
     bogotest = 0;
@@ -193,16 +203,7 @@ void process_parameters(int argc, char **argv, bool warn_on_error)
 
     /* directories from command line and config file are already handled */
 
-    if (set_wordlist_dir(NULL, PR_ENV_BOGO) != 0 &&
-	set_wordlist_dir(NULL, PR_ENV_HOME) != 0) {
-	fprintf(stderr, "Can't find HOME or BOGOFILTER_DIR in environment.\n");
-	exit(EX_ERROR);
-    }
-
-    if (!config_setup) {
-	/* set up default wordlist */
-	init_wordlist("word", WORDLIST, 0, WL_REGULAR);
-    }
+    wordlists_set_bogohome();
 
     stats_prefix= stats_in_header ? "  " : "# ";
 
@@ -310,6 +311,7 @@ static const char *help_text[] = {
     "info options:\n",
     "  -t, --terse               - set terse output mode.\n",
     "  -T, --fixed-terse-format  - set invariant terse output mode.\n",
+    "  -q, --quiet               - suppress token statistics.\n",
     "  -U, --report-unsure       - print statistics if spamicity is 'unsure'.\n",
     "  -v, --verbosity           - set debug verbosity level.\n",
     "  -y, --timestamp-date      - set date for token timestamps.\n",
@@ -342,6 +344,9 @@ static const char *help_text[] = {
     "  --terse-format                    short form\n",
     "  --thresh-update                   no update if near 0 or 1\n",
     "  --timestamp                       apply token timestamps\n",
+#ifndef	DISABLE_UNICODE
+    "  --unicode=yes/no                  enable/disable unicode based wordlist\n",
+#endif
     "  --unsure-subject-tag              like spam-subject-tag\n",
     "  --user-config-file                configuration file\n",
     "  --wordlist                        specify wordlist parameters\n",
@@ -353,25 +358,26 @@ static const char *help_text[] = {
     NULL
 };
 
-static void help(void)
+static void help(FILE *fp)
 {
     uint i;
-    (void)fprintf(stderr,
+    (void)fprintf(fp,
                   "%s version %s\n"
                   "\n"
-                  "Usage:  %s [options] < message\n",
+                  "Usage:  %s [options] < message\n"
+                  "\n",
                   progtype, version, PACKAGE
 	);
     for (i=0; help_text[i] != NULL; i++)
-	(void)fprintf(stderr, "%s", help_text[i]);
+	(void)fprintf(fp, "%s", help_text[i]);
 }
 
 static void print_version(void)
 {
-    (void)fprintf(stderr,
+    (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2004 Eric S. Raymond,\n"
+		  "Copyright (C) 2002-2005 Eric S. Raymond,\n"
 		  "    David Relson, Matthias Andree, Greg Louis\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
@@ -506,7 +512,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'h':
-	help();
+	help(stdout);
 	exit(EX_OK);
 
     case 'I':
@@ -543,6 +549,10 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	passthrough = true;
 	break;
 
+    case 'q':
+	quiet = true;
+	break;
+
     case 'Q':
 	if (pass == PASS_1_CLI)
 	    query += 1;
@@ -629,7 +639,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	if (pass != PASS_1_CLI) {
 	    comma_parse(option, val, &min_dev, &robs, &robx);
 	    if (DEBUG_CONFIG(1))
-		printf("md %6.4f, rs %6.4f, rx %6.4f\n", min_dev, robs, robx);
+		fprintf(dbgout, "md %6.4f, rs %6.4f, rx %6.4f\n", min_dev, robs, robx);
 	}
 	break;
 
@@ -652,7 +662,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	if (pass != PASS_1_CLI) {
 	    comma_parse(option, val, &spam_cutoff, &ham_cutoff, NULL);
 	    if (DEBUG_CONFIG(1))
-		printf("sc %6.4f, hc %6.4f\n", spam_cutoff, ham_cutoff);
+		fprintf(dbgout, "sc %6.4f, hc %6.4f\n", spam_cutoff, ham_cutoff);
 	}
 	break;
 
@@ -696,12 +706,15 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     case O_THRESH_UPDATE:		get_double(name, val, &thresh_update);			break;
     case O_TIMESTAMP:			timestamp_tokens = get_bool(name, val);			break;
     case O_UNSURE_SUBJECT_TAG:		unsure_subject_tag = get_string(name, val);		break;
+    case O_UNICODE:			encoding = get_bool(name, val) ? E_UNICODE : E_RAW;	break;
     case O_WORDLIST:			configure_wordlist(val);				break;
 
-    case O_DB_TRANSACTION:		fTransaction = get_bool(name, val);			break;
+    case O_DB_TRANSACTION:		eTransaction = get_txn(name, val);			break;
 
     default:
+#ifndef	DISABLE_TRANSACTIONS
 	if (!dsm_options_bogofilter(option, name, val))
+#endif
 	    abort();
     }
 }
@@ -725,6 +738,7 @@ rc_t query_config(void)
     Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "sp_esf", sp_esf, sp_esf);
     Q1 fprintf(stdout, "\n");
     Q1 fprintf(stdout, "%-17s = %s\n",    "block-on-subnets",    YN(block_on_subnets));
+    Q1 fprintf(stdout, "%-17s = %s\n",    "encoding",		 (encoding != E_UNICODE) ? "raw" : "utf-8");
     Q1 fprintf(stdout, "%-17s = %s\n",    "charset-default",     charset_default);
     Q1 fprintf(stdout, "%-17s = %s\n",    "replace-nonascii-characters", YN(replace_nonascii_characters));
     Q2 fprintf(stdout, "%-17s = %s\n",    "no-header-tags",      YN(header_line_markup));
@@ -732,7 +746,7 @@ rc_t query_config(void)
     Q2 fprintf(stdout, "%-17s = %s\n",    "report-unsure",       YN(unsure_stats));
     Q1 fprintf(stdout, "%-17s = %0.6f\n", "thresh-update",       thresh_update);
     Q1 fprintf(stdout, "%-17s = %s\n",    "timestamp",           YN(timestamp_tokens));
-    Q2 fprintf(stdout, "%-17s = %d\n",    "timestamp-date",      today);
+    Q2 fprintf(stdout, "%-17s = %ld\n",   "timestamp-date",      (long int)today);
     Q1 fprintf(stdout, "\n");
     Q1 fprintf(stdout, "%-17s = %s\n", "spam-header-name",    spam_header_name);
     Q1 fprintf(stdout, "%-17s = %s\n", "spam-subject-tag",    NB(spam_subject_tag));
@@ -752,18 +766,20 @@ rc_t query_config(void)
     Q2 fprintf(stdout, "%-18s = %s\n", "user-config-file", NB(user_config_file));
     Q2 fprintf(stdout, "\n");
 
-    Q2 fprintf(stdout, "%-18s = %s\n", "bogofilter-dir", 	bogohome);
+    Q2 fprintf(stdout, "%-18s = %s\n", "bogofilter-dir",        bogohome);
     Q2 display_wordlists(word_lists, "%-18s   ");
     Q2 fprintf(stdout, "\n");
 
-    Q2 fprintf(stdout, "%-18s = %d\n", "db-cachesize",        	db_cachesize);
+#ifndef	DISABLE_TRANSACTIONS
+    Q2 fprintf(stdout, "%-18s = %lu\n", "db-cachesize",         (unsigned long)db_cachesize);
 
+#ifdef	ENABLE_TRANSACTIONS
 #ifdef	HAVE_DECL_DB_CREATE
-    Q2 fprintf(stdout, "%-18s = %d\n", "db-lk-max-locks",   	db_max_locks);
-    Q2 fprintf(stdout, "%-18s = %d\n", "db-lk-max-objects", 	db_max_objects);
     Q2 fprintf(stdout, "%-18s = %s\n", "db-log-autoremove",     YN(db_log_autoremove));
 #ifdef	FUTURE_DB_OPTIONS
-    Q2 fprintf(stdout, "%-18s = %s\n", "db-log-txn-durable",	YN(db_txn_durable));
+    Q2 fprintf(stdout, "%-18s = %s\n", "db-log-txn-durable",    YN(db_txn_durable));
+#endif
+#endif
 #endif
 #endif
 
