@@ -1,4 +1,4 @@
-/* $Id: bogofilter.c,v 1.53 2005/03/16 01:59:38 relson Exp $ */
+/* $Id: bogofilter.c,v 1.54 2005/05/19 11:46:05 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -109,7 +109,7 @@ rc_t bogofilter(int argc, char **argv)
 	    wordhash_add(words, w, &wordprop_init);
 
 	if (classify_msg || write_msg) {
-	    lookup_words(w);
+	    lookup_words(w);			/* This reads the database */
 	    spamicity = msg_compute_spamicity(w, NULL);
 	    status = msg_status();
 	    if (run_type & RUN_UPDATE)		/* Note: don't register if RC_UNSURE */
