@@ -1,4 +1,4 @@
-/* $Id: bogotune.c,v 1.210 2005/04/07 00:01:51 relson Exp $ */
+/* $Id: bogotune.c,v 1.223 2005/06/30 00:07:59 relson Exp $ */
 
 /*****************************************************************************
 
@@ -148,6 +148,7 @@ static word_t *w_msg_count;
 
 static uint message_count;
 
+static wordhash_t *train;
 static tunelist_t *ns_and_sp;
 static tunelist_t *ns_msglists, *sp_msglists;
 
@@ -652,24 +653,33 @@ static void load_wordlist(ds_foreach_t *hook, void *userdata)
 static int load_hook(word_t *key, dsv_t *data, void *userdata)
 /* returns 0 if ok, 1 if not ok */
 {
-    wordhash_t *train = userdata;
-
     wordprop_t *tokenprop = wordhash_insert(train, key, sizeof(wordprop_t), &wordprop_init);
+
+    (void) userdata;	/* quiet compiler complaint */
+
     tokenprop->cnts.bad = data->spamcount;
     tokenprop->cnts.good = data->goodcount;
 
     if (word_cmps(key, ".MSG_COUNT") == 0)
 	set_msg_counts(data->goodcount, data->spamcount);
 
+    if (word_cmps(key, ".ENCODING") == 0) {
+	if (encoding == E_UNKNOWN)
+	    encoding = data->spamcount;
+	if (encoding != data->spamcount) {
+	    fprintf(stderr, "Can't mix database encodings, i.e. utf-8 and any other.\n");
+	    exit(EX_ERROR);
+	}
+    }
+
     return 0;
 }
 
-static void set_train_msg_counts(wordhash_t *train, wordhash_t *wh)
+static void set_train_msg_counts(wordhash_t *wh)
 {
     wordprop_t *count;
     count = wordhash_insert(wh, w_msg_count, sizeof(wordprop_t), NULL);
-    if (count->cnts.good == 0 || count->cnts.bad == 0)
-	load_wordlist(load_hook, train);
+
     if (msgs_good == 0 && msgs_bad == 0) {
 	fprintf(stderr, "Can't find '.MSG_COUNT'.\n");
 	exit(EX_ERROR);
@@ -689,7 +699,6 @@ static void print_msgcount_entry(const char *token, uint bad, uint good)
 static void write_msgcount_file(wordhash_t *wh)
 {
     hashnode_t *hn;
-    wordhash_t *train = ns_and_sp->train;
 
     print_msgcount_entry(".MSG_COUNT", msgs_bad, msgs_good);
 
@@ -714,8 +723,6 @@ static void write_msgcount_file(wordhash_t *wh)
 
 static uint read_mailbox(char *arg, mlhead_t *msgs)
 {
-    wordhash_t *train = ns_and_sp->train;
-
     if (verbose) {
 	printf("Reading %s\n", arg);
 	fflush(stdout);
@@ -731,7 +738,7 @@ static uint read_mailbox(char *arg, mlhead_t *msgs)
 	collect_words(whc);
 
 	if (ds_path != NULL && (msgs_good + msgs_bad) == 0)
-	    set_train_msg_counts(train, whc);
+	    set_train_msg_counts(whc);
 
 	if (whc->count == 0 && !quiet) {
 	    printf("msg #%u, count is %u\n", message_count, whc->count);
@@ -799,7 +806,6 @@ static void distribute(int mode, tunelist_t *ns_or_sp)
 
     mlitem_t *item;
     mlhead_t *msgs = ns_or_sp->msgs;
-    wordhash_t *train = ns_and_sp->train;
 
     int score_count = 0;
     int train_count = 0;
@@ -873,7 +879,7 @@ static void print_version(void)
     (void)fprintf(stderr,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2004 Greg Louis, David Relson\n\n"
+		  "Copyright (C) 2002-2005 Greg Louis, David Relson\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -1299,6 +1305,7 @@ static void bogotune_init(void)
 {
     const char *msg_count = MSG_COUNT;
     w_msg_count = word_news(msg_count);
+    train       = wordhash_new();
     ns_and_sp   = tunelist_new("tr");		/* training lists */
     ns_msglists = tunelist_new("ns");		/* non-spam scoring lists */
     sp_msglists = tunelist_new("sp");		/* spam     scoring lists */
@@ -1391,6 +1398,18 @@ static void show_elapsed_time(int beg, int end, uint cnt, double val,
 	       MIN(tm), SECONDS(tm), cnt, lbl1, val, lbl2);
 }
 
+static rc_t bogolex(void)
+{
+    rc_t status = RC_OK;
+
+    if (!check_msgcount_parms())
+	exit(EX_ERROR);
+
+    read_mailbox(bogolex_file, NULL);
+
+    return status;
+}
+
 static rc_t bogotune(void)
 {
     bool skip;
@@ -1400,16 +1419,6 @@ static rc_t bogotune(void)
     uint cnt, scan;
     rc_t status = RC_OK;
 
-    bogotune_init();
-
-    if (bogolex_file != NULL) {
-	if (!check_msgcount_parms())
-	    exit(EX_ERROR);
-	load_wordlist(load_hook, ns_and_sp->train);
-	read_mailbox(bogolex_file, NULL);
-	return status;
-    }
-
     beg = time(NULL);
 
     ham_cutoff = 0.0;
@@ -1510,8 +1519,8 @@ static rc_t bogotune(void)
 	fprintf(stderr, "Warning:  Using ESF values (sp=%8.6f, ns=%8.6f) from config file.\n", sp_esf, ns_esf);
 
     /* No longer needed */
-    wordhash_free(ns_and_sp->train);
-    ns_and_sp->train = NULL;
+    wordhash_free(train);
+    train = NULL;
 
     for (scan=0; scan <= 1 && !skip; scan ++) {
 	uint r_count;
@@ -1751,7 +1760,19 @@ int main(int argc, char **argv) /*@globals errno,stderr,stdout@*/
 	init_wordlist("word", ds_path, 0, WL_REGULAR);
     }
 
-    bogotune();
+    bogotune_init();
+
+    if (ds_flag == DS_DSK)
+	load_wordlist(load_hook, train);
+
+    /* if encoding not yet set, assume old style */
+    if (encoding == E_UNKNOWN)
+	encoding = E_RAW;
+
+    if (bogolex_file != NULL)
+	bogolex();
+    else
+	bogotune();
 
     bogotune_free();
 
