@@ -1,4 +1,4 @@
-/* $Id: debug.h,v 1.11 2004/10/14 11:33:20 relson Exp $ */
+/* $Id: debug.h,v 1.15 2005/03/12 04:23:29 relson Exp $ */
 
 /*****************************************************************************
 
@@ -10,23 +10,30 @@ NAME:
 #ifndef _DEBUG_H
 #define _DEBUG_H
 
+/* define FILE type */
+#include <stdio.h>
+/* define other types */
+#include "bftypes.h"
+
 #define	DEBUG_NONE	0
 
 #define	MASK_BIT(uc)	( 1 << (uc - 'A'))
 
-#define BIT_NAMES	"abcdfghlmstw"
+#define BIT_NAMES	"abcdghilmrstwy"
 #define BIT_ALGORITHM	MASK_BIT('A')
 #define BIT_READER	MASK_BIT('B')
 #define BIT_CONFIG	MASK_BIT('C')
 #define BIT_DATABASE	MASK_BIT('D')
 #define BIT_GENERAL	MASK_BIT('G')
 #define BIT_HTML	MASK_BIT('H')
+#define BIT_ICONV	MASK_BIT('I')
 #define BIT_LEXER	MASK_BIT('L')
 #define BIT_MIME	MASK_BIT('M')
 #define BIT_REGISTER	MASK_BIT('R')
 #define BIT_SPAMICITY	MASK_BIT('S')
 #define BIT_TEXT	MASK_BIT('T')
 #define BIT_WORDLIST	MASK_BIT('W')
+#define BIT_MEMORY	MASK_BIT('Y')
 
 extern FILE	 *dbgout;
 extern u_int32_t  debug_mask;
@@ -37,7 +44,9 @@ extern u_int32_t  debug_mask;
 #define DEBUG_DATABASE(level)	0
 #define DEBUG_ALGORITHM(level)	0
 #define DEBUG_HTML(level)	0
+#define DEBUG_ICONV(level)	0
 #define DEBUG_LEXER(level)	0
+#define DEBUG_MEMORY(level)	0
 #define DEBUG_MIME(level)	0
 #define DEBUG_READER(level)	0
 #define DEBUG_REGISTER(level)	0
@@ -50,7 +59,9 @@ extern u_int32_t  debug_mask;
 #define DEBUG_DATABASE(level)	((debug_mask & BIT_DATABASE)  && (verbose > level))
 #define DEBUG_ALGORITHM(level)	((debug_mask & BIT_ALGORITHM) && (verbose > level))
 #define DEBUG_HTML(level)	((debug_mask & BIT_HTML)      && (verbose > level))
+#define DEBUG_ICONV(level)	((debug_mask & BIT_ICONV)     && (verbose > level))
 #define DEBUG_LEXER(level)	((debug_mask & BIT_LEXER)     && (verbose > level))
+#define DEBUG_MEMORY(level)	((debug_mask & BIT_MEMORY)    && (verbose > level))
 #define DEBUG_MIME(level)	((debug_mask & BIT_MIME)      && (verbose > level))
 #define DEBUG_READER(level)	((debug_mask & BIT_READER)    && (verbose > level))
 #define DEBUG_REGISTER(level)	((debug_mask & BIT_REGISTER)  && (verbose > level))
