@@ -1,4 +1,4 @@
-/* $Id: longoptions.h,v 1.11 2005/03/23 23:57:14 m-a Exp $ */
+/* $Id: longoptions.h,v 1.18 2005/06/18 16:36:09 relson Exp $ */
 
 /*****************************************************************************
 
@@ -31,6 +31,8 @@ typedef enum longopts_e {
     O_DB_MAX_OBJECTS,
     O_DB_MAX_LOCKS,
     O_DB_CHECKPOINT,
+    O_DB_LIST_LOGFILES,
+    O_DB_PRINT_LEAFPAGE_COUNT,
     O_DB_PRINT_PAGESIZE,
     O_DB_PRUNE,
     O_DB_RECOVER,
@@ -60,17 +62,26 @@ typedef enum longopts_e {
     O_TERSE_FORMAT,
     O_THRESH_UPDATE,
     O_TIMESTAMP,
+    O_UNICODE,
     O_UNSURE_SUBJECT_TAG,
     O_USER_CONFIG_FILE,
     O_WORDLIST
 } longopts_t;
 
+#ifndef	DISABLE_UNICODE
+#define	UNICODE_OPTION	\
+    { "unicode",			R, 0, O_UNICODE },
+#else
+#define	UNICODE_OPTION
+#endif
+
 /* common options */
 #define LONGOPTIONS_COMMON \
     { "config-file",			R, 0, O_CONFIG_FILE }, \
     { "no-config-file",			N, 0, 'C' }, \
     { "help",				N, 0, 'h' }, \
     { "input-file",			N, 0, 'I' }, \
+       UNICODE_OPTION \
     { "version",			N, 0, 'V' }, \
     { "verbosity",			N, 0, 'v' },
 
@@ -82,6 +93,9 @@ typedef enum longopts_e {
     { "user-config-file",		R, 0, O_USER_CONFIG_FILE }, \
     { "replace-nonascii-characters",	R, 0, O_REPLACE_NONASCII_CHARACTERS },
 
+/* options for bogolexer and bogoutil */
+#define LONGOPTIONS_LEX_UTIL
+
 /* options for bogofilter and bogoutil - some preprocessor workarounds here */
 #define lo1
 #define lo2
