@@ -1,4 +1,4 @@
-/* $Id: prob.h,v 1.1 2003/12/28 03:52:41 relson Exp $ */
+/* $Id: prob.h,v 1.5 2005/01/17 03:00:48 relson Exp $ */
 
 /*****************************************************************************
 
@@ -13,7 +13,8 @@ AUTHOR:
 #ifndef	PROB_H
 #define	PROB_H
 
-extern double calc_prob(uint good, uint bad);
+/** calculate the probability that a token is bad */
+extern double calc_prob(uint good, uint bad, uint goodmsgs, uint badmsgs);
 
 #endif	/* PROB_H */
 
