@@ -1,4 +1,4 @@
-/* $Id: token.c,v 1.120 2005/04/05 11:29:50 relson Exp $ */
+/* $Id: token.c,v 1.121 2005/05/25 23:33:57 relson Exp $ */
 
 /*****************************************************************************
 
@@ -27,7 +27,7 @@ AUTHOR:
 #include "token.h"
 #include "xmemrchr.h"
 
-#define	MAX_PREFIX_LEN 	  5		/* maximim length of prefix     */
+#define	MAX_PREFIX_LEN 	  5		/* maximum length of prefix     */
 #define	MSG_COUNT_PADDING 2 * 10	/* space for 2 10-digit numbers */
 
 /* Local Variables */
